#!/usr/bin/python
# Copyright (C) 2011 by Eric Price <ecprice@mit.edu>
#
# Distributed under the MIT X11 license.
# (i.e. no restrictions on use other than keeping the copyright notice.)
#
# Known issues: not clever when there are phonemes that could be replaced.

"""
Search for pairs of words that match up to phonemes.

Example:
$ python phonemes.py "K W" "W"
[...]
QUIZZED => WHIZZED
QUIZZES => WHIZZES
[...]

Expects a phonetic dictionary in the format of the CMU
pronouncing dictionary:
  http://www.speech.cs.cmu.edu/cgi-bin/cmudict
As of this writing, the appropriate file is at
  https://cmusphinx.svn.sourceforge.net/svnroot/cmusphinx/trunk/cmudict/cmudict.0.7a

"""

import argparse
import re

parser = argparse.ArgumentParser(formatter_class=argparse.RawDescriptionHelpFormatter,
                                 description=__doc__)
parser.add_argument('--dictionary', metavar="FNAME",
                    type=str,
                    help='Location of CMU dictionary file.',
                    default='cmudict.0.7a')
parser.add_argument(metavar='FROM', dest="from_", type=str, nargs=1,
                    help='Phonemes to convert from')
parser.add_argument(metavar="TO", dest='to', type=str, nargs=1,
                    help='Phonemes to convert to')
parser.add_argument('-a', '--replace_all', action='store_const',
                    const=-1, default=1, dest='max_replace',
                    help='Replace all instances of phoneme, not just one.')

args = parser.parse_args()
old = args.from_[0].upper()
new = args.to[0].upper()

lines = [line for line in open(args.dictionary).read().splitlines()
         if not line.startswith(';;;')]

phonemes_to_words = {}
pairs = []

for line in lines:
    word, phonemes = line.split('  ')
    word = re.sub('\([0-9]+\)', '', word)
    pairs.append((word, phonemes))
    phonemes_to_words.setdefault(phonemes, []).append(word)

for word, phonemes in pairs:
    if old in phonemes:
        newp = phonemes.replace(old, new, args.max_replace)
        if newp in phonemes_to_words:
            print word, '=>', ' '.join(phonemes_to_words[newp])
